@@ -1,7 +1,7 @@
 /*
 * libslack - http://libslack.org/
 *
-* Copyright (C) 1999-2010 raf <raf@raf.org>
+* Copyright (C) 1999-2002, 2004, 2010, 2020 raf <raf@raf.org>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,9 @@
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
-* along with this program; if not, write to the Free Software
-* Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-* or visit http://www.gnu.org/copyleft/gpl.html
+* along with this program; if not, see <https://www.gnu.org/licenses/>.
 *
-* 20100612 raf <raf@raf.org>
+* 20201111 raf <raf@raf.org>
 */
 
 #ifndef LIBSLACK_FIO_H
@@ -38,6 +36,8 @@ int rw_timeout(int fd, long sec, long usec);
 int nap(long sec, long usec);
 int fcntl_set_flag(int fd, int flag);
 int fcntl_clear_flag(int fd, int flag);
+int fcntl_set_fdflag(int fd, int flag);
+int fcntl_clear_fdflag(int fd, int flag);
 int fcntl_lock(int fd, int cmd, int type, int whence, int start, int len);
 int nonblock_set(int fd, int arg);
 int nonblock_on(int fd);
